@@ -1,9 +1,11 @@
-/* $Id: appointment.c,v 1.18 2003/12/08 09:44:02 twogood Exp $ */
+/* $Id: appointment.c,v 1.34 2005/05/20 11:08:38 twogood Exp $ */
 #define _GNU_SOURCE 1
 #include "appointment.h"
 #include "appointment_ids.h"
 #include "generator.h"
 #include "parser.h"
+#include "strbuf.h"
+#include "strv.h"
 #include <rapi.h>
 #include <synce_log.h>
 #include <string.h>
@@ -20,6 +22,18 @@
 #define MINUTES_PER_DAY (24*60)
 #define SECONDS_PER_DAY (MINUTES_PER_DAY*60)
 
+#define BLOB0067_STR  "BLOB0067="
+
+#if 0
+static const uint8_t BLOB0067_HEADER[20] = { 
+  0x04, 0x00, 0x00, 0x00, 
+  0x82, 0x00, 0xe0, 0x00,
+  0x74, 0xc5, 0xb7, 0x10,
+  0x1a, 0x82, 0xe0, 0x08,
+  0x00, 0x00, 0x00, 0x00 
+};
+#endif
+
 typedef struct _EventGeneratorData
 {
   CEPROPVAL* start;
@@ -27,6 +41,10 @@ typedef struct _EventGeneratorData
   CEPROPVAL* type;
   CEPROPVAL* reminder_minutes;
   CEPROPVAL* reminder_enabled;
+#if ENABLE_RECURRENCE
+  CEPROPVAL* recurrence_pattern;
+  CEPROPVAL* unique;
+#endif
 } EventGeneratorData;
 
 /*
@@ -88,6 +106,17 @@ static bool on_propval_reminder_minutes(Generator* g, CEPROPVAL* propval, void*
   return true;
 }
 
+static bool on_propval_recurrence_pattern(Generator* g, CEPROPVAL* propval, void* cookie)
+{
+#if ENABLE_RECURRENCE
+  EventGeneratorData* data = (EventGeneratorData*)cookie;
+  data->recurrence_pattern = propval;
+#else
+  synce_warning("Recurrence support not enabled");
+#endif
+  return true;
+}
+
 static bool on_propval_start(Generator* g, CEPROPVAL* propval, void* cookie)
 {
   EventGeneratorData* data = (EventGeneratorData*)cookie;
@@ -95,6 +124,15 @@ static bool on_propval_start(Generator* g, CEPROPVAL* propval, void* cookie)
   return true;
 }
 
+#if ENABLE_RECURRENCE
+bool on_propval_unique(Generator* g, CEPROPVAL* propval, void* cookie)
+{
+  EventGeneratorData* data = (EventGeneratorData*)cookie;
+  data->unique = propval;
+  return true;
+}
+#endif
+
 bool rra_appointment_to_vevent(/*{{{*/
     uint32_t id,
     const uint8_t* data,
@@ -126,6 +164,7 @@ bool rra_appointment_to_vevent(/*{{{*/
     goto exit;
 
   generator_add_property(generator, ID_BUSY_STATUS, on_propval_busy_status);
+  generator_add_property(generator, ID_CATEGORIES,  on_propval_categories);
   generator_add_property(generator, ID_DURATION,    on_propval_duration);
   generator_add_property(generator, ID_APPOINTMENT_TYPE, on_propval_type);
   generator_add_property(generator, ID_LOCATION,    on_propval_location);
@@ -134,7 +173,11 @@ bool rra_appointment_to_vevent(/*{{{*/
   generator_add_property(generator, ID_REMINDER_ENABLED, on_propval_reminder_enabled);
   generator_add_property(generator, ID_SENSITIVITY, on_propval_sensitivity);
   generator_add_property(generator, ID_APPOINTMENT_START,       on_propval_start);
+  generator_add_property(generator, ID_RECURRENCE_PATTERN, on_propval_recurrence_pattern);
   generator_add_property(generator, ID_SUBJECT,     on_propval_subject);
+#if ENABLE_RECURRENCE
+  generator_add_property(generator, ID_UNIQUE,      on_propval_unique);
+#endif
 
   if (!generator_set_data(generator, data, data_size))
     goto exit;
@@ -174,10 +217,12 @@ bool rra_appointment_to_vevent(/*{{{*/
     time_t end_time = 0;
     const char* type = NULL;
     const char* format = NULL;
-    
+    struct tm* (*xtime)(const time_t *timep) = NULL;
+
     switch (event_generator_data.type->val.lVal)
     {
       case APPOINTMENT_TYPE_ALL_DAY:
+        xtime  = localtime;
         type   = "DATE";
         format = "%Y%m%d";
 
@@ -189,8 +234,9 @@ bool rra_appointment_to_vevent(/*{{{*/
 
 
       case APPOINTMENT_TYPE_NORMAL:
+        xtime  = gmtime;
         type   = "DATE-TIME";
-        if (tzi)
+        if (!tzi)
           format = "%Y%m%dT%H%M%SZ";
         else
           format = "%Y%m%dT%H%M%S";
@@ -201,8 +247,8 @@ bool rra_appointment_to_vevent(/*{{{*/
 
         if (tzi)
         {
-          start_time = rra_timezone_convert_to_utc(tzi, start_time);
-          end_time   = rra_timezone_convert_to_utc(tzi, end_time);
+          start_time = rra_timezone_convert_from_utc(tzi, start_time);
+          end_time   = rra_timezone_convert_from_utc(tzi, end_time);
         }
       
         break;
@@ -215,12 +261,12 @@ bool rra_appointment_to_vevent(/*{{{*/
 
     if (type && format)
     {
-      strftime(buffer, sizeof(buffer), format, localtime(&start_time));
+      strftime(buffer, sizeof(buffer), format, xtime(&start_time));
       generator_add_with_type(generator, "DTSTART", type, buffer);
       
       if (end_time)
       {
-        strftime(buffer, sizeof(buffer), format, localtime(&end_time));
+        strftime(buffer, sizeof(buffer), format, xtime(&end_time));
         generator_add_with_type(generator, "DTEND",   type, buffer);
       }
     }
@@ -260,11 +306,66 @@ bool rra_appointment_to_vevent(/*{{{*/
     generator_add_simple(generator, "END", "VALARM");
   }
 
-  generator_add_simple(generator, "END", "VEVENT");
+#if ENABLE_RECURRENCE
+  if (event_generator_data.recurrence_pattern)
+  {
+    if (!recurrence_generate_rrule(generator, event_generator_data.recurrence_pattern))
+      synce_warning("Failed to generate RRULE from recurrence pattern.");
+
+    if (event_generator_data.unique)
+    {
+      char* buffer = NULL;
+      unsigned i;
+      bool is_text = true;
+
+      assert(CEVT_BLOB == (event_generator_data.unique->propid & 0xffff));
+
+      for (i = 0; i < event_generator_data.unique->val.blob.dwCount; i++)
+        if (!isprint(event_generator_data.unique->val.blob.lpb[i]))
+        {
+          is_text = false;
+          break;
+        }
+
+      if (is_text)
+      {
+        /* This is if SynCE has saved a text UID to this BLOB */
+        buffer = malloc(event_generator_data.unique->val.blob.dwCount + 1);
+        memcpy(
+            buffer, 
+            event_generator_data.unique->val.blob.lpb, 
+            event_generator_data.unique->val.blob.dwCount);
+        buffer[event_generator_data.unique->val.blob.dwCount] = '\0';
+      }
+      else
+      {
+        /* This is for the usual binary UID */
+        char* p = NULL;
+        buffer = (char*)malloc(
+            event_generator_data.unique->val.blob.dwCount * 2 + sizeof(BLOB0067_STR));
+
+        strcpy(buffer, BLOB0067_STR);
+        p = buffer + strlen(buffer);
+
+        for (i = 0; i < event_generator_data.unique->val.blob.dwCount; i++)
+        {
+          sprintf(p, "%02x", event_generator_data.unique->val.blob.lpb[i]);
+          p += 2;
+        }
+      }
+
+      generator_add_simple(generator, "UID", buffer);
+      free(buffer);
+    }
+  }
+#endif
+
+   generator_add_simple(generator, "END", "VEVENT");
 #if 0
   generator_add_simple(generator, "END", "VCALENDAR");
 #endif
   
+ 
   if (!generator_get_result(generator, vevent))
     goto exit;
 
@@ -288,9 +389,19 @@ typedef struct _EventParserData
   bool has_alarm;
   mdir_line* dtstart;
   mdir_line* dtend;
+#if ENABLE_RECURRENCE
+  RRA_MdirLineVector* exdates;
   mdir_line* rrule;
+  mdir_line* uid;
+#endif
 } EventParserData;
 
+static bool on_timezone_tzid(Parser* p, mdir_line* line, void* cookie)
+{
+  synce_trace("TZID = '%s'", line->values[0]);
+  return true;
+}
+
 static bool on_alarm_trigger(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
   EventParserData* event_parser_data = (EventParserData*)cookie;
@@ -349,26 +460,39 @@ exit:
   return true;
 }/*}}}*/
 
-static bool on_mdir_line_dtend(Parser* p, mdir_line* line, void* cookie)
+static bool on_mdir_line_dtend(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
   EventParserData* event_parser_data = (EventParserData*)cookie;
   event_parser_data->dtend = line;
   return true;
-}
+}/*}}}*/
 
-static bool on_mdir_line_dtstart(Parser* p, mdir_line* line, void* cookie)
+static bool on_mdir_line_dtstart(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
   EventParserData* event_parser_data = (EventParserData*)cookie;
   event_parser_data->dtstart = line;
   return true;
-}
+}/*}}}*/
 
-static bool on_mdir_line_rrule(Parser* p, mdir_line* line, void* cookie)
+#if ENABLE_RECURRENCE
+static bool on_mdir_line_exdate(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
   EventParserData* event_parser_data = (EventParserData*)cookie;
-  event_parser_data->rrule = line;
+  rra_mdir_line_vector_add(event_parser_data->exdates, line);
   return true;
-}
+}/*}}}*/
+#endif
+
+static bool on_mdir_line_rrule(Parser* p, mdir_line* line, void* cookie)/*{{{*/
+{
+#if ENABLE_RECURRENCE
+  EventParserData* event_parser_data = (EventParserData*)cookie;
+  event_parser_data->rrule = line;
+#else
+  synce_warning("Recurrence support not enabled");
+#endif
+  return true;
+}/*}}}*/
 
 static bool on_mdir_line_transp(Parser* p, mdir_line* line, void* cookie)/*{{{*/
 {
@@ -381,6 +505,15 @@ static bool on_mdir_line_transp(Parser* p, mdir_line* line, void* cookie)/*{{{*/
   return true;
 }/*}}}*/
 
+#if ENABLE_RECURRENCE
+static bool on_mdir_line_uid(Parser* parser, mdir_line* line, void* cookie)/*{{{*/
+{
+  EventParserData* event_parser_data = (EventParserData*)cookie;
+  event_parser_data->uid = line;
+  return true;
+}/*}}}*/
+#endif
+
 bool rra_appointment_from_vevent(/*{{{*/
     const char* vevent,
     uint32_t* id,
@@ -395,9 +528,14 @@ bool rra_appointment_from_vevent(/*{{{*/
   ParserComponent* calendar;
   ParserComponent* event;
   ParserComponent* alarm;
+  ParserComponent* timezone;
   int parser_flags = 0;
   EventParserData event_parser_data;
   memset(&event_parser_data, 0, sizeof(EventParserData));
+  
+#if ENABLE_RECURRENCE
+  event_parser_data.exdates = rra_mdir_line_vector_new();
+#endif
 
   switch (flags & RRA_APPOINTMENT_CHARSET_MASK)
   {
@@ -411,6 +549,10 @@ bool rra_appointment_from_vevent(/*{{{*/
       break;
   }
 
+  timezone = parser_component_new("vTimeZone");
+  parser_component_add_parser_property(timezone, 
+      parser_property_new("tzid", on_timezone_tzid));
+  
   alarm = parser_component_new("vAlarm");
 
   parser_component_add_parser_property(alarm, 
@@ -419,6 +561,8 @@ bool rra_appointment_from_vevent(/*{{{*/
   event = parser_component_new("vEvent");
   parser_component_add_parser_component(event, alarm);
 
+  parser_component_add_parser_property(event, 
+      parser_property_new("Categories", on_mdir_line_categories));
   parser_component_add_parser_property(event, 
       parser_property_new("Class", on_mdir_line_class));
   parser_component_add_parser_property(event, 
@@ -427,6 +571,10 @@ bool rra_appointment_from_vevent(/*{{{*/
       parser_property_new("dtEnd", on_mdir_line_dtend));
   parser_component_add_parser_property(event, 
       parser_property_new("dtStart", on_mdir_line_dtstart));
+#if ENABLE_RECURRENCE
+  parser_component_add_parser_property(event, 
+      parser_property_new("exDate", on_mdir_line_exdate));
+#endif
   parser_component_add_parser_property(event, 
       parser_property_new("Location", on_mdir_line_location));
   parser_component_add_parser_property(event, 
@@ -435,9 +583,14 @@ bool rra_appointment_from_vevent(/*{{{*/
       parser_property_new("Summary", on_mdir_line_summary));
   parser_component_add_parser_property(event, 
       parser_property_new("Transp", on_mdir_line_transp));
+#if ENABLE_RECURRENCE
+  parser_component_add_parser_property(event, 
+      parser_property_new("UId", on_mdir_line_uid));
+#endif
 
   calendar = parser_component_new("vCalendar");
   parser_component_add_parser_component(calendar, event);
+  parser_component_add_parser_component(calendar, timezone);
 
   /* allow parsing to start with either vCalendar or vEvent */
   base = parser_component_new(NULL);
@@ -470,7 +623,7 @@ bool rra_appointment_from_vevent(/*{{{*/
       synce_error("Failed add time from line");
       goto exit;
     }
-    
+
     if (event_parser_data.dtend)
     {
       time_t start = 0;
@@ -503,16 +656,56 @@ bool rra_appointment_from_vevent(/*{{{*/
     }
 
 #if ENABLE_RECURRENCE
-    if (!recurrence_parse_rrule(
-          parser, 
-          event_parser_data.rrule, 
-          event_parser_data.dtstart,
-          event_parser_data.dtend))
-    {
-      synce_error("Failed to parse recurrence rule");
-      goto exit;
+    if (event_parser_data.rrule)
+    { 
+      if (!recurrence_parse_rrule(
+            parser, 
+            event_parser_data.dtstart,
+            event_parser_data.dtend,
+            event_parser_data.rrule, 
+            event_parser_data.exdates))
+        synce_warning("Failed to parse recurrence rule");
+
+      if (event_parser_data.uid)
+      {
+        if (0 == strncmp(event_parser_data.uid->values[0], BLOB0067_STR, strlen(BLOB0067_STR)))
+        {
+          /* A binary UID from SynCE */
+          size_t size = (strlen(event_parser_data.uid->values[0]) - strlen(BLOB0067_STR)) / 2;
+          uint8_t* buffer = malloc(size);
+          unsigned i;
+          char* p;
+
+          p = event_parser_data.uid->values[0] + strlen(BLOB0067_STR);
+
+          for (i = 0; i < size; i++, p+=2)
+          {
+            char tmp[3] = {p[0], p[1], '\0'};
+            buffer[i] = (uint8_t)strtol(tmp, NULL, 16);
+          }
+
+          parser_add_blob(parser, ID_UNIQUE, buffer, size);
+          free(buffer);
+        }
+        else
+        {
+          /* A text UID */
+          parser_add_blob(
+              parser, 
+              ID_UNIQUE, 
+              (uint8_t*)event_parser_data.uid->values[0], 
+              strlen(event_parser_data.uid->values[0]));
+        }
+      }
     }
+    else
 #endif
+      parser_add_int16(parser, ID_OCCURENCE, OCCURENCE_ONCE);
+  }
+  else
+  {
+    synce_error("No DTSTART found");
+    goto exit;
   }
 
   if (!event_parser_data.has_alarm)
@@ -536,13 +729,22 @@ bool rra_appointment_from_vevent(/*{{{*/
  	success = true;
 
 exit:
+#if ENABLE_RECURRENCE
+  rra_mdir_line_vector_destroy(event_parser_data.exdates, true);
+#endif
   /* destroy components (the order is important!) */
   parser_component_destroy(base);
   parser_component_destroy(calendar);
   parser_component_destroy(event);
   parser_component_destroy(alarm);
+  parser_component_destroy(timezone);
   parser_destroy(parser);
+
+  if (!success)
+  {
+    synce_trace("Failure on this vEvent: '%s'", vevent);
+  }
+  
 	return success;
 }/*}}}*/
 
-
