@@ -1,9 +1,10 @@
-/* $Id: contact.c,v 1.15 2003/12/08 09:44:02 twogood Exp $ */
+/* $Id: contact.c,v 1.22 2005/05/13 11:35:28 twogood Exp $ */
 #define _GNU_SOURCE 1
 #include "contact.h"
 #include "contact_ids.h"
 #include "strbuf.h"
 #include "dbstream.h"
+#include "strv.h"
 #include <rapi.h>
 #include <synce_log.h>
 #include <stdio.h>
@@ -13,6 +14,8 @@
 #include <assert.h>
 #include "internal.h"
 
+#define VERBOSE 0
+
 static const char* product_id = "-//SYNCE RRA//NONSGML Version 1//EN";
 
 /* 
@@ -120,6 +123,20 @@ static void strbuf_append_tel_type(StrBuf* strbuf, const char* value, uint32_t f
 	strbuf_append_type(strbuf, "TEL", value, flags);
 }
 
+static void strbuf_append_date(StrBuf* strbuf, const char* name, FILETIME* filetime)
+{
+  char buffer[12];
+  TIME_FIELDS time_fields;
+
+  time_fields_from_filetime(filetime, &time_fields);
+  snprintf(buffer, sizeof(buffer), "%04i-%02i-%02i", time_fields.Year, time_fields.Month, time_fields.Day);
+
+  strbuf_append(strbuf, name);
+  strbuf_append_c(strbuf, ':');
+  strbuf_append(strbuf, buffer);
+  strbuf_append_crlf(strbuf);
+}
+
 static bool rra_contact_to_vcard2(/*{{{*/
 		uint32_t id, 
 		CEPROPVAL* pFields, 
@@ -157,6 +174,13 @@ static bool rra_contact_to_vcard2(/*{{{*/
 	WCHAR* work_postal_code = NULL;
 	WCHAR* work_country = NULL;
 
+	/* other address parts */
+	WCHAR* other_street = NULL;
+	WCHAR* other_locality = NULL;
+	WCHAR* other_region = NULL;
+	WCHAR* other_postal_code = NULL;
+	WCHAR* other_country = NULL;
+
 	strbuf_append(vcard, "BEGIN:vCard\r\n");
 
 	strbuf_append(vcard, "VERSION:");
@@ -189,6 +213,14 @@ static bool rra_contact_to_vcard2(/*{{{*/
 		/* TODO: validate data types */
 		switch (pFields[i].propid >> 16)
 		{
+      case ID_ANNIVERSARY:
+        strbuf_append_date(vcard, "X-EVOLUTION-ANNIVERSARY", &pFields[i].val.filetime);
+        break;
+
+      case ID_BIRTHDAY:
+        strbuf_append_date(vcard, "BDAY", &pFields[i].val.filetime);
+        break;
+      
 #if 0
 			case ID_NOTE:
 				{
@@ -358,6 +390,26 @@ static bool rra_contact_to_vcard2(/*{{{*/
 				work_country = pFields[i].val.lpwstr;
 				break;
 
+			case ID_OTHER_STREET:
+				other_street = pFields[i].val.lpwstr;
+				break;
+
+			case ID_OTHER_LOCALITY:
+				other_locality = pFields[i].val.lpwstr;
+				break;
+
+			case ID_OTHER_REGION:
+				other_region = pFields[i].val.lpwstr;
+				break;
+
+			case ID_OTHER_POSTAL_CODE:
+				other_postal_code = pFields[i].val.lpwstr;
+				break;
+
+			case ID_OTHER_COUNTRY:
+				other_country = pFields[i].val.lpwstr;
+				break;
+
 			case ID_EMAIL:
 				strbuf_append_type(vcard, "EMAIL", "INTERNET,PREF", flags);
 				strbuf_append_escaped_wstr(vcard, pFields[i].val.lpwstr, flags);
@@ -371,6 +423,36 @@ static bool rra_contact_to_vcard2(/*{{{*/
 				strbuf_append_crlf(vcard);
 				break;
 
+			case ID_SPOUSE:
+				strbuf_append(vcard, "X-EVOLUTION-SPOUSE:");
+				strbuf_append_escaped_wstr(vcard, pFields[i].val.lpwstr, flags);
+				strbuf_append_crlf(vcard);
+				break;
+
+			case ID_ASSISTANT:
+				strbuf_append(vcard, "X-EVOLUTION-ASSISTANT:");
+				strbuf_append_escaped_wstr(vcard, pFields[i].val.lpwstr, flags);
+				strbuf_append_crlf(vcard);
+				break;
+
+			case ID_ASSISTANT_TEL:
+				strbuf_append_tel_type(vcard, "X-EVOLUTION-ASSISTANT", flags);
+				strbuf_append_escaped_wstr(vcard, pFields[i].val.lpwstr, flags);
+				strbuf_append_crlf(vcard);
+				break;
+
+			case ID_OFFICE_LOC:
+				strbuf_append(vcard, "X-EVOLUTION-OFFICE:");
+				strbuf_append_escaped_wstr(vcard, pFields[i].val.lpwstr, flags);
+				strbuf_append_crlf(vcard);
+				break;
+
+			case ID_RADIO_TEL:
+				strbuf_append_tel_type(vcard, "X-EVOLUTION-RADIO", flags);
+				strbuf_append_escaped_wstr(vcard, pFields[i].val.lpwstr, flags);
+				strbuf_append_crlf(vcard);
+				break;
+
 			default:
 				synce_warning("Did not handle field with ID %04x", pFields[i].propid >> 16);
 				break;
@@ -450,6 +532,25 @@ static bool rra_contact_to_vcard2(/*{{{*/
 		strbuf_append_crlf      (vcard);
 	}
 
+	if (other_street || other_locality || other_postal_code || other_country)
+	{
+		strbuf_append_type(vcard, "ADR", "POSTAL", flags);
+		strbuf_append_escaped_wstr (vcard, NULL, flags); /* post office box */
+		strbuf_append_c            (vcard, ';');
+		strbuf_append_escaped_wstr (vcard, NULL, flags); /* extended address */
+		strbuf_append_c            (vcard, ';');
+		strbuf_append_escaped_wstr (vcard, other_street, flags);
+		strbuf_append_c            (vcard, ';');
+		strbuf_append_escaped_wstr (vcard, other_locality, flags);
+		strbuf_append_c            (vcard, ';');
+		strbuf_append_escaped_wstr (vcard, other_region, flags); /* region */
+		strbuf_append_c            (vcard, ';');
+		strbuf_append_escaped_wstr (vcard, other_postal_code, flags);
+		strbuf_append_c            (vcard, ';');
+		strbuf_append_escaped_wstr (vcard, other_country, flags);
+		strbuf_append_crlf      (vcard);
+	}
+
 
 	if (!have_fn)
 	{
@@ -568,18 +669,19 @@ static uint32_t name_ids[NAME_FIELD_COUNT] =
 
 #define HOME 0
 #define WORK 1
+#define OTHER 2
 
 #define ADDRESS_FIELD_COUNT 7
 
-static uint32_t address_ids[ADDRESS_FIELD_COUNT][2] = 
+static uint32_t address_ids[ADDRESS_FIELD_COUNT][3] = 
 {
 	{0, 0}, /* post office box */
 	{0, 0}, /* extended address */
-	{ID_HOME_STREET,      ID_WORK_STREET},      /* street */
-	{ID_HOME_LOCALITY,    ID_WORK_LOCALITY},    /* locality */
-	{ID_HOME_REGION,      ID_WORK_REGION},      /* region */
-	{ID_HOME_POSTAL_CODE, ID_WORK_POSTAL_CODE}, /* postal code */
-	{ID_HOME_COUNTRY,     ID_WORK_COUNTRY}      /* country */
+	{ID_HOME_STREET,      ID_WORK_STREET,        ID_OTHER_STREET},      /* street */
+	{ID_HOME_LOCALITY,    ID_WORK_LOCALITY,      ID_OTHER_LOCALITY},    /* locality */
+	{ID_HOME_REGION,      ID_WORK_REGION,        ID_OTHER_REGION},      /* region */
+	{ID_HOME_POSTAL_CODE, ID_WORK_POSTAL_CODE,   ID_OTHER_POSTAL_CODE}, /* postal code */
+	{ID_HOME_COUNTRY,     ID_WORK_COUNTRY,       ID_OTHER_COUNTRY}      /* country */
 };
 
 static char* strdup_quoted_printable(const char* source)/*{{{*/
@@ -650,6 +752,39 @@ static void unescape_string(char* value)/*{{{*/
 	*dest = '\0';
 }/*}}}*/
 
+static bool date_to_struct(const char* datetime, TIME_FIELDS* time_fields)/*{{{*/
+{
+  int count;
+  memset(time_fields, 0, sizeof(*time_fields));
+
+  count = sscanf(datetime, "%4hd-%2hd-%2hd", 
+      &time_fields->Year,
+      &time_fields->Month,
+      &time_fields->Day);
+
+  if (count != 3)
+  {
+    synce_error("Bad date: '%s'", datetime);
+    return false;
+  }
+
+  return true;
+}/*}}}*/
+
+static void add_date(Parser* parser, uint32_t id, const char* type, char* value)
+{
+  TIME_FIELDS time_fields;
+
+  assert(value);
+
+  if (date_to_struct(value, &time_fields))
+  {
+    CEPROPVAL* propval = &parser->fields[parser->field_index++];
+    propval->propid = (id << 16) | CEVT_FILETIME;
+    time_fields_to_filetime(&time_fields, &propval->val.filetime);
+  }
+}
+
 static void add_string(Parser* parser, uint32_t id, const char* type, char* value)/*{{{*/
 {
 	char* converted = NULL;
@@ -679,51 +814,6 @@ static void add_string(Parser* parser, uint32_t id, const char* type, char* valu
 		free(converted);
 }/*}}}*/
 
-static char** strsplit(const char* source, int separator)/*{{{*/
-{
-	int i;
-	int count = 0;
-	const char* p = NULL;
-	char** result = NULL;
-	size_t length = 0;
-
-	for (p = source; *p; p++)
-		if (separator == *p)
-			count++;
-
-	result = malloc((count + 2) * sizeof(char*));
-
-	for (p = source, i = 0; i < count; i++)
-	{
-		length = strchr(p, separator) - p;
-		result[i] = strndup(p, length);
-		p += length + 1;
-	}
-
-	result[i++] = strdup(p);
-
-	result[i] = NULL;
-	return result;
-}/*}}}*/
-
-static void strv_dump(char** strv)/*{{{*/
-{
-	char** pp;
-
-	for (pp = strv; *pp; pp++)
-		synce_trace("'%s'", *pp);
-}/*}}}*/
-
-static void strv_free(char** strv)/*{{{*/
-{
-	char** pp;
-
-	for (pp = strv; *pp; pp++)
-		free(*pp);
-
-	free(strv);
-}/*}}}*/
-
 static bool parser_handle_field(/*{{{*/
 		Parser* parser,
 		char* name, 
@@ -732,8 +822,10 @@ static bool parser_handle_field(/*{{{*/
 {
 	bool success = false;
 
+#if VERBOSE
 	synce_trace("Found field '%s' with type '%s' and contents '%s'",
 			name, type, value);
+#endif
 
 	if (STR_EQUAL(name, "BEGIN"))/*{{{*/
 	{
@@ -794,12 +886,10 @@ static bool parser_handle_field(/*{{{*/
 
 		for (i = 0; i < NAME_FIELD_COUNT && name[i]; i++)
 		{
-			if (name_ids[i] && *name[i])
+			if (*name[i])
 			{
 				add_string(parser, name_ids[i], type, name[i]);
 			}
-			else
-				break;
 		}
 
 		strv_free(name);
@@ -810,7 +900,9 @@ static bool parser_handle_field(/*{{{*/
 		char** address = strsplit(value, ';');
 		int where;
 
-		if (STR_IN_STR(type, "WORK"))
+		if (STR_IN_STR(type, "POSTAL"))
+		        where = OTHER;
+		else if (STR_IN_STR(type, "WORK"))
 			where = WORK;
 		else if (STR_IN_STR(type, "HOME"))
 			where = HOME;
@@ -849,6 +941,14 @@ static bool parser_handle_field(/*{{{*/
 		{
 			add_string(parser, ID_MOBILE_TEL, type, value);
 		}
+		else if (STR_IN_STR(type, "X-EVOLUTION-ASSISTANT"))
+		{
+			add_string(parser, ID_ASSISTANT_TEL, type, value);
+		}
+		else if (STR_IN_STR(type, "X-EVOLUTION-RADIO"))
+		{
+			add_string(parser, ID_RADIO_TEL, type, value);
+		}
 		else
 		{
 			synce_trace("Type '%s' for field '%s' not recognized.",
@@ -874,9 +974,11 @@ static bool parser_handle_field(/*{{{*/
 	else if (STR_EQUAL(name, "ORG"))/*{{{*/
 	{
 		char* separator = strchr(value, ';');
-		if (separator && separator[1])
+		if (separator)
 		{
-			add_string(parser, ID_DEPARTMENT, type, separator + 1);
+			if (separator[1]) {
+				add_string(parser, ID_DEPARTMENT, type, separator + 1);
+			}
 			*separator = '\0';
 		}
 
@@ -891,7 +993,9 @@ static bool parser_handle_field(/*{{{*/
 	}/*}}}*/
 	else if (STR_EQUAL(name, "X-EVOLUTION-FILE-AS"))/*{{{*/
 	{
+#if VERBOSE
 		synce_trace("So, your contact has been in Evolution?");
+#endif
 	}/*}}}*/
 	else if (STR_EQUAL(name, "UID"))/*{{{*/
 	{
@@ -905,6 +1009,27 @@ static bool parser_handle_field(/*{{{*/
   {
     add_string(parser, ID_CATEGORY, type, value);
   }
+  else if (STR_EQUAL(name, "BDAY"))
+  {
+    add_date(parser, ID_BIRTHDAY, type, value);
+  }
+  else if (STR_EQUAL(name, "X-EVOLUTION-ANNIVERSARY"))
+  {
+    add_date(parser, ID_ANNIVERSARY, type, value);
+  }
+  else if (STR_EQUAL(name, "X-EVOLUTION-SPOUSE"))
+  {
+    add_string(parser, ID_SPOUSE, type, value);
+  }
+  else if (STR_EQUAL(name, "X-EVOLUTION-ASSISTANT"))
+  {
+    add_string(parser, ID_ASSISTANT, type, value);
+  }
+  else if (STR_EQUAL(name, "X-EVOLUTION-OFFICE"))
+  {
+    add_string(parser, ID_OFFICE_LOC, type, value);
+  }
+
 #if 0
 	else if (STR_EQUAL(name, ""))
 	{
