@@ -11,18 +11,28 @@
 #define ID_COMPANY      0x3a16
 #define ID_JOB_TITLE    0x3a17
 #define ID_DEPARTMENT   0x3a18
+#define ID_OFFICE_LOC   0x3a19
 #define ID_MOBILE_TEL   0x3a1c
+#define ID_RADIO_TEL    0x3a1d
 #define ID_CAR_TEL      0x3a1e
 #define ID_WORK_FAX     0x3a24
 #define ID_HOME_FAX     0x3a25
 #define ID_HOME2_TEL    0x3a2f
 
-#define ID_CATEGORY     0x4005
+#define ID_BIRTHDAY      0x4001
+#define ID_ANNIVERSARY   0x4003
+#define ID_CATEGORY      0x4005
+
+#define ID_ASSISTANT     0x4002
+#define ID_ASSISTANT_TEL 0x4004
+
+#define ID_CHILDREN     0x4006
 
 #define ID_WORK2_TEL    0x4007
 #define ID_WEB_PAGE     0x4008
 #define ID_PAGER        0x4009
-
+#define ID_SPOUSE       0x400a
+ 
 #define ID_FULL_NAME    0x4013
 #define ID_TITLE        0x4023
 #define ID_MIDDLE_NAME  0x4024
@@ -39,6 +49,12 @@
 #define ID_WORK_POSTAL_CODE  0x4048
 #define ID_WORK_COUNTRY      0x4049
 
+#define ID_OTHER_STREET      0x404a
+#define ID_OTHER_LOCALITY    0x404b
+#define ID_OTHER_REGION      0x404c
+#define ID_OTHER_POSTAL_CODE 0x404d
+#define ID_OTHER_COUNTRY     0x404e
+
 #define ID_EMAIL        0x4083
 #define ID_EMAIL2       0x4093
 #define ID_EMAIL3       0x40a3
