@@ -1,4 +1,4 @@
-/* $Id: rrac.c,v 1.20 2003/12/22 08:07:17 twogood Exp $ */
+/* $Id: rrac.c,v 1.24 2004/08/01 18:02:51 twogood Exp $ */
 #include "rrac.h"
 #include "rrac_packet.h"
 #include <synce_log.h>
@@ -18,6 +18,7 @@
 #define LE32(p)    letoh32(*(uint32_t*)p)
 
 #define DUMP_PACKETS 0
+#define VERBOSE 0
 
 #if DUMP_PACKETS
 #define DUMP(desc,data,len) dump(desc, data, len)
@@ -109,19 +110,22 @@ bool rrac_event_pending(RRAC* self)
   return false;
 }
 
-bool rrac_event_wait(RRAC* self, int timeoutInSeconds)
+bool rrac_event_wait(RRAC* self, int timeoutInSeconds, bool* gotEvent)
 {
-  if (self)
+  if (self && self->cmd_socket)
   {
     short events = EVENT_READ;
     /*synce_trace("Wating for event");*/
     if (synce_socket_wait(self->cmd_socket, timeoutInSeconds, &events))
-      return (events & EVENT_READ) != 0;
+    {
+      *gotEvent = (events & EVENT_READ) != 0;
+      return true;
+    }
     else
       synce_error("synce_socket_wait failed");
   }
   else
-    synce_error("RRAC pointer is NULL");
+    synce_error("RRAC pointer or command socket is NULL");
 
   return false;
 }
@@ -184,20 +188,40 @@ void rrac_disconnect(RRAC* rrac)/*{{{*/
 
 bool rrac_is_connected(RRAC* rrac)
 {
-  return rrac && rrac->data_socket;
+#if 0
+  return rrac && rrac->cmd_socket && rrac->data_socket;
+#else
+  bool result = false;
+  
+  if (rrac)
+  {
+    short events = EVENT_READ | EVENT_WRITE;
+    if (synce_socket_wait(rrac->cmd_socket, 0, &events))
+    {
+      if (!(events & EVENT_ERROR))
+        result = true;
+    }
+
+    if (!result)
+      rrac_disconnect(rrac);
+  }
+
+  return result;
+#endif
 }
 
 static bool rrac_recv_any(RRAC* rrac, CommandHeader* header, uint8_t** data)/*{{{*/
 {
 	bool success = false;
+  bool got_event = false;
 
 	*data = NULL;
 
-  if (!rrac_event_wait(rrac, RRAC_TIMEOUT))
+  if (!rrac_event_wait(rrac, RRAC_TIMEOUT, &got_event) || !got_event)
   {
- 		synce_error("No data received in %i seconds!", RRAC_TIMEOUT);
-		goto exit;
- }
+    synce_error("No data received in %i seconds!", RRAC_TIMEOUT);
+    goto exit;
+  }
 
 	if (!synce_socket_read(rrac->cmd_socket, header, sizeof(CommandHeader)))
 	{
@@ -531,7 +555,7 @@ exit:
 bool rrac_recv_reply_6f_c1(/*{{{*/
 		RRAC* rrac,
 		RawObjectType** object_type_array,
-		size_t* object_type_count)
+		uint32_t* object_type_count)
 {
 	bool success = false;
 	uint8_t* data = NULL;
@@ -846,9 +870,7 @@ bool rrac_recv_data(/*{{{*/
     synce_trace("chunk_size = %04x, aligned_size = %04x, stuff = %04x",
         chunk_header.size, aligned_size, chunk_header.stuff);
 
-    if ((unsigned)((chunk_header.stuff & 0xc) >> 2) == (aligned_size - chunk_header.size))
-      synce_trace("Flags and sizes match");
-    else
+    if ((unsigned)((chunk_header.stuff & 0xc) >> 2) != (aligned_size - chunk_header.size))
       synce_warning("Flags and sizes do not match!");
  
 		if (!synce_socket_read(rrac->data_socket, *data + total_size, aligned_size))
@@ -927,8 +949,10 @@ bool rrac_send_data(/*{{{*/
 			chunk_header.stuff = htole16(stuff);
     }
 
+#if VERBOSE
     synce_trace("chunk_size = %04x, aligned_size = %04x, stuff = %04x",
         chunk_size, aligned_size, chunk_header.stuff);
+#endif
         
 		DUMP("chunk header", &chunk_header, sizeof(chunk_header));
 
@@ -948,7 +972,9 @@ bool rrac_send_data(/*{{{*/
 		if (aligned_size > chunk_size)
 		{
 			char pad[3] = {0,0,0};
+#if VERBOSE
 			synce_trace("Writing %i bytes padding", aligned_size - chunk_size);
+#endif
 			if (!synce_socket_write(rrac->data_socket, pad, aligned_size - chunk_size))
 			{
 				synce_error("Failed to write padding");
@@ -1143,7 +1169,7 @@ bool sync_command_notify_ids(SyncCommand* self, uint32_t* ids)
 
     total = LE32(p);  p += sizeof(uint32_t);
 
-    synce_trace("total = %08x", total);
+    /* synce_trace("total = %08x", total); */
 
     if (self->size < (16 + total))
     {
@@ -1167,6 +1193,39 @@ exit:
   return success;
 }
 
+bool sync_command_negotiation_get(
+    SyncCommand* self, 
+    SyncNegotiation* negotiation)
+{
+  bool success = false;
+
+  if (self && negotiation)
+  {
+    uint8_t* p = self->data;
+
+    if (self->size < 0x10)
+    {
+      synce_error("Invalid packet.");
+      goto exit;
+    }
+
+    if (self->size > 0x10)
+      synce_warning("Unexpected packet size %08x", self->size);
+
+    negotiation->type_id  = LE32(p);  p += sizeof(uint32_t);
+    negotiation->old_id   = LE32(p);  p += sizeof(uint32_t);
+    negotiation->new_id   = LE32(p);  p += sizeof(uint32_t);
+    negotiation->flags    = LE32(p);  p += sizeof(uint32_t);
+
+    success = true;
+  }
+  else
+    synce_error("Invalid parameters to function.");
+
+exit:
+  return success;
+}
+
 SyncCommand* rrac_recv_command(RRAC* self)/*{{{*/
 {
   SyncCommand* result = NULL;
